/*
 * Detects_Pressed_USW_and_Beep.c
 *
 *  Created on: 2025/01/27
 *      Author: egaono-electronics
 */

#include "Detects_Pressed_USW_and_Beep.h"

#include "Detects_Pressed_USW.h"
#include "Generate_Beep.h"



//---------------------------------------------------------
// USERスイッチ押下検出とBeep
//----------------------------------------------------------
// 引数:
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ


// 戻り値:
//   -1 : 処理中
//    0 : OK終了
//    1 : NG
//----------------------------------------------------------
int16_t Detects_Pressed_USW_and_Beep(uint8_t *puint8_JobNum)
{
  //----------------------------------------------------
  // 呼び出す関数の引数に使用する処理番号は、
  // 必ず static の変数宣言をしてください。
  //----------------------------------------------------
  static uint8_t STC_uint8_JobNum_USW;   // USW処理番号
  static uint8_t STC_uint8_JobNum_Beep;  // Beep処理番号


  int16_t int16_Return;


  switch(*puint8_JobNum){
  case 0:
    STC_uint8_JobNum_USW = 0;   // USW処理番号初期化
    STC_uint8_JobNum_Beep = 0;  // Beep処理番号初期化


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //-----------------------------------------------------------
    // USERスイッチ押下検出
    //-----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納された変数のポインタ


    // 戻り値：
    //   -1 : 検出なし
    //    0 : 検出あり
    //-----------------------------------------------------------
    int16_Return = def_M_Detect_USW(&STC_uint8_JobNum_USW);
    if(int16_Return == -1){
      // 検出なし
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //----------------------------------------------------------
    // Beepを発生する。
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // int16_t int16_Time_Beep : Beepを発生する時間


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //----------------------------------------------------------
    int16_Return = Generate_Beep(&STC_uint8_JobNum_Beep,
                                 (int16_t)def_Time_Beep);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    *puint8_JobNum = 1;  // USERスイッチ検出処理に移行する。: case 1:
                         // case 1: と case 2: を繰り返し実行する。
    int16_Return = 0;    // OK : 処理が1回終了
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG終了
    break;
  }

  return(int16_Return);
}
